use std::sync::Arc;

use async_sqlx_session::PostgresSessionStore;
use como_core::{items::DynItemService, projects::DynProjectService, users::DynUserService};
use tracing::log::info;

use crate::{
    configs::AppConfig,
    database::ConnectionPool,
    services::{
        item_service::MemoryItemService, project_service::MemoryProjectService,
        user_service::DefaultUserService,
    },
};

#[derive(Clone)]
pub struct ServiceRegister {
    pub item_service: DynItemService,
    pub project_service: DynProjectService,
    pub user_service: DynUserService,
    pub session_store: PostgresSessionStore,
}

impl ServiceRegister {
    pub async fn new(pool: ConnectionPool, config: Arc<AppConfig>) -> anyhow::Result<Self> {
        info!("creating services");

        let item_service = Arc::new(MemoryItemService::new()) as DynItemService;
        let project_service = Arc::new(MemoryProjectService::new()) as DynProjectService;
        let user_service = Arc::new(DefaultUserService::new(pool.clone())) as DynUserService;
        let store = PostgresSessionStore::new(&config.database_url).await?;

        store.migrate().await?;

        info!("services created succesfully");

        Ok(Self {
            item_service,
            user_service,
            project_service,
            session_store: store,
        })
    }
}
